import { Variable, VariableSource, Accessor } from './lib/variable';
import { EnvVarError } from './lib/env-error';
import { accessors } from './lib/accessors';
import { EnvLogger, createLogger } from './lib/logger';
export { VariableSource, EnvLogger, EnvVarError, accessors, createLogger, Accessor };
export declare type EnvVarInstanceParams = {
    variables: VariableSource;
    requiredByDefault?: false;
    logger?: EnvLogger;
};
/**
 * Creates an EnvVarInstance with custom variables. Supports parameters that
 * can make all variables required by default, and a logging function.
 *
 * ```
 * import { from } from 'env-var';
 *
 * const get = from({
 *   // In a Node.js environment you might simply pass process.env here, but
 *   // in a Next.js or React application you need to explicitly reference the
 *   // variables as shown
 *   variables: {
 *     REACT_APP_BASE_URL: process.env.REACT_APP_BASE_URL
 *   },
 *
 *   // Setting requiredByDefault to true means it's not necessary to call
 *   // required() on all variables. This can be bypassed by calling
 *   // `required(false)` when reading a variable
 *   requiredByDefault: true,
 *
 *   // Print env-var logs if trace logging is enabled
 *   logger: (varname, msg) => log.trace(`${varname}: ${msg}`)
 * })
 *
 * const url = get('BASE_URL).asUrlString()
 * ```
 *
 * @param {EnvVarInstanceParams} params
 */
export declare function from<K extends EnvVarInstanceParams, T extends K['requiredByDefault']>(params: K): EnvVarInstance<T extends true ? undefined : unknown, K>;
/**
 * Read a variable from the environment.
 *
 * Reads from the global process.env a server-side Node.js environment. All
 * other environments require creating an EnvVarInstance first, i.e:
 *
 * ```
 * import { from } from 'env-var';
 *
 * const get = from({
 *   variables: {
 *     // For example, in React an explicit reference to
 *     // process.env.REACT_APP_BASE_URL is required
 *     REACT_APP_BASE_URL: process.env.REACT_APP_BASE_URL
 *   }
 * })
 *
 * const url = get('BASE_URL).asUrlString()
 * ```
 *
 * @param {String} varname
 */
export declare function get(varname?: string): Variable<undefined>;
/**
 * An EnvVarInstance is typically created using the `from()` function. This
 * class is exposed for convenience, but isn't recommended for direct use.
 */
export declare class EnvVarInstance<VariableSource, T extends EnvVarInstanceParams> {
    private config;
    constructor(config: T);
    get<V extends VariableSource>(): V;
    get(variableName: Extract<keyof T['variables'], string>): Variable;
    get(variableName?: Extract<keyof T['variables'], string>): Variable;
}
