"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EnvVarInstance = exports.get = exports.from = exports.createLogger = exports.accessors = exports.EnvVarError = void 0;
var variable_1 = require("./lib/variable");
var env_error_1 = require("./lib/env-error");
Object.defineProperty(exports, "EnvVarError", { enumerable: true, get: function () { return env_error_1.EnvVarError; } });
var accessors_1 = require("./lib/accessors");
Object.defineProperty(exports, "accessors", { enumerable: true, get: function () { return accessors_1.accessors; } });
var logger_1 = require("./lib/logger");
Object.defineProperty(exports, "createLogger", { enumerable: true, get: function () { return logger_1.createLogger; } });
var check_environment_1 = require("./lib/check-environment");
/**
 * Creates an EnvVarInstance with custom variables. Supports parameters that
 * can make all variables required by default, and a logging function.
 *
 * ```
 * import { from } from 'env-var';
 *
 * const get = from({
 *   // In a Node.js environment you might simply pass process.env here, but
 *   // in a Next.js or React application you need to explicitly reference the
 *   // variables as shown
 *   variables: {
 *     REACT_APP_BASE_URL: process.env.REACT_APP_BASE_URL
 *   },
 *
 *   // Setting requiredByDefault to true means it's not necessary to call
 *   // required() on all variables. This can be bypassed by calling
 *   // `required(false)` when reading a variable
 *   requiredByDefault: true,
 *
 *   // Print env-var logs if trace logging is enabled
 *   logger: (varname, msg) => log.trace(`${varname}: ${msg}`)
 * })
 *
 * const url = get('BASE_URL).asUrlString()
 * ```
 *
 * @param {EnvVarInstanceParams} params
 */
function from(params) {
    if (!params || !params.variables) {
        throw new env_error_1.EnvVarError('Since v8.0.0, from() parameters must be an object that contains a "variables" object. The "variables" should be key-value pairs where all values are of type string or undefined.');
    }
    var e = new EnvVarInstance(params);
    return e;
}
exports.from = from;
/**
 * Read a variable from the environment.
 *
 * Reads from the global process.env a server-side Node.js environment. All
 * other environments require creating an EnvVarInstance first, i.e:
 *
 * ```
 * import { from } from 'env-var';
 *
 * const get = from({
 *   variables: {
 *     // For example, in React an explicit reference to
 *     // process.env.REACT_APP_BASE_URL is required
 *     REACT_APP_BASE_URL: process.env.REACT_APP_BASE_URL
 *   }
 * })
 *
 * const url = get('BASE_URL).asUrlString()
 * ```
 *
 * @param {String} varname
 */
function get(varname) {
    if (!(0, check_environment_1.isNode)()) {
        throw new env_error_1.EnvVarError('The exported get() function is only supported in Node.js environments. For all other environments use from() to create an EnvVarInstance first.');
    }
    if (arguments.length > 1) {
        throw new env_error_1.EnvVarError('It looks like you passed more than one argument to env.get(). Since env-var@6.0.0 this is no longer supported. To set a default value use env.get(TARGET).default(DEFAULT)');
    }
    return new EnvVarInstance({
        variables: process.env
    }).get(varname);
}
exports.get = get;
/**
 * An EnvVarInstance is typically created using the `from()` function. This
 * class is exposed for convenience, but isn't recommended for direct use.
 */
var EnvVarInstance = /** @class */ (function () {
    function EnvVarInstance(config) {
        this.config = config;
    }
    EnvVarInstance.prototype.get = function (variableName) {
        if (!variableName) {
            return this.config.variables;
        }
        if (arguments.length > 1) {
            throw new env_error_1.EnvVarError('It looks like you passed more than one argument to env.get(). Since env-var@6.0.0 this is no longer supported. To set a default value use env.get(TARGET).default(DEFAULT)');
        }
        return new variable_1.Variable(this.config.variables, variableName, this.config.logger ? this.config.logger : function () { }, undefined, this.config.requiredByDefault);
    };
    return EnvVarInstance;
}());
exports.EnvVarInstance = EnvVarInstance;
