import { EnvVarError } from './env-error';
import { EnvLogger } from './logger';
export declare type VariableSource = Record<string, string | undefined>;
export declare type Accessor<ReturnType, ArgsType = undefined> = (v: string, error: (s: string) => EnvVarError, args?: ArgsType) => ReturnType;
export declare class Variable<T = undefined> {
    protected container: VariableSource;
    protected varName: string;
    protected logger: EnvLogger;
    protected isBase64?: boolean | undefined;
    protected isRequired?: boolean | undefined;
    protected defValue?: string | undefined;
    protected exampleValue?: string | undefined;
    protected descriptionValue?: string | undefined;
    constructor(container: VariableSource, varName: string, logger: EnvLogger, isBase64?: boolean | undefined, isRequired?: boolean | undefined, defValue?: string | undefined, exampleValue?: string | undefined, descriptionValue?: string | undefined);
    /**
     * Logs the given string using the provided logger
     * @param {String} str
     * @param {String} str
     */
    protected log(str: string): void;
    /**
     * Instructs env-var to first convert the value of the variable from base64
     * when reading it using a function such as asString()
     */
    convertFromBase64(): this;
    /**
     * Set a default value for the variable
     * @param {String} value
     */
    default(value: string | number | unknown): this;
    /**
     * Set an example value for this variable. If the variable value is not set
     * or is set to an invalid value this example will be show in error output.
     * @param {String} example
     */
    example(ex: string): this;
    /**
     * Set a description. This is used to provide a more informative error
     * message if the variable is missing or set to an invalid value.
     * @param description
     * @returns
     */
    description(description: string): this;
    /**
     * Throw an error with a consistent type/format.
     * @param {String} value
     */
    protected createError(msg: string): EnvVarError;
    protected getValue<V>(parser: (value: string) => V): V | T;
    usingAccessor<ReturnType, ArgsType>(ext: Accessor<ReturnType, ArgsType>, args?: ArgsType): T | ReturnType;
    /**
     * Ensures a variable is set in the given environment container. Throws an
     * EnvVarError if the variable is not set or a default is not provided
     * @param {Boolean} required
     */
    required(required?: boolean): Variable<never>;
    required(required: true): Variable<never>;
    required(required: false): Variable<undefined>;
    asString(): string | T;
    asArray(delimiter?: string): string[] | T;
    asBoolStrict(): boolean | T;
    asBool(): boolean | T;
    asEnum<V extends string>(validValues: V[]): V | T;
    asFloat(): number | T;
    asFloatNegative(): number | T;
    asFloatPositive(): number | T;
    asInt(): number | T;
    asIntNegative(): number | T;
    asIntPositive(): number | T;
    asJson(): unknown[] | Record<string, unknown> | T;
    asJsonArray(): unknown[] | T;
    asJsonObject(): Record<string, unknown> | T;
    asPortNumber(): number | T;
    asRegExp(flags?: string): RegExp | T;
    asUrlObject(): URL | T;
    asUrlString(): string | T;
}
