'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Variable = void 0;
var accessors_1 = require("./accessors");
var env_error_1 = require("./env-error");
var base64Regex = /^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$/;
var Variable = /** @class */ (function () {
    function Variable(container, varName, logger, isBase64, isRequired, defValue, exampleValue, descriptionValue) {
        this.container = container;
        this.varName = varName;
        this.logger = logger;
        this.isBase64 = isBase64;
        this.isRequired = isRequired;
        this.defValue = defValue;
        this.exampleValue = exampleValue;
        this.descriptionValue = descriptionValue;
    }
    /**
     * Logs the given string using the provided logger
     * @param {String} str
     * @param {String} str
     */
    Variable.prototype.log = function (str) {
        this.logger(this.varName, str);
    };
    /**
     * Instructs env-var to first convert the value of the variable from base64
     * when reading it using a function such as asString()
     */
    Variable.prototype.convertFromBase64 = function () {
        this.log('marking for base64 conversion');
        this.isBase64 = true;
        return this;
    };
    /**
     * Set a default value for the variable
     * @param {String} value
     */
    Variable.prototype.default = function (value) {
        if (typeof value === 'number') {
            this.defValue = value.toString();
        }
        else if (Array.isArray(value) || (typeof value === 'object' && value !== null)) {
            this.defValue = JSON.stringify(value);
        }
        else if (typeof value !== 'string') {
            throw new env_error_1.EnvVarError('values passed to default() must be of Number, String, Array, or Object type');
        }
        else {
            this.defValue = value;
        }
        this.log("setting default value to \"".concat(this.defValue, "\""));
        return this;
    };
    /**
     * Set an example value for this variable. If the variable value is not set
     * or is set to an invalid value this example will be show in error output.
     * @param {String} example
     */
    Variable.prototype.example = function (ex) {
        this.exampleValue = ex;
        return this;
    };
    /**
     * Set a description. This is used to provide a more informative error
     * message if the variable is missing or set to an invalid value.
     * @param description
     * @returns
     */
    Variable.prototype.description = function (description) {
        this.descriptionValue = description;
        return this;
    };
    /**
     * Throw an error with a consistent type/format.
     * @param {String} value
     */
    Variable.prototype.createError = function (msg) {
        var errMsg = "\"".concat(this.varName, "\" ").concat(msg);
        if (this.descriptionValue) {
            errMsg = "".concat(errMsg, ". ").concat(this.descriptionValue);
        }
        if (this.exampleValue) {
            errMsg = "".concat(errMsg, ". An example of a valid value would be: ").concat(this.exampleValue);
        }
        return new env_error_1.EnvVarError(errMsg);
    };
    Variable.prototype.getValue = function (parser) {
        var value = this.container[this.varName];
        var defValue = this.defValue;
        this.log("will be read from the environment");
        if (typeof value === 'undefined') {
            if (typeof defValue === 'undefined' && this.isRequired) {
                this.log('was not found in the environment, but is required to be set');
                // Var is not set, nor is a default. Throw an error
                throw this.createError('is a required variable, but it was not set');
            }
            else if (typeof defValue !== 'undefined') {
                this.log("was not found in the environment, parsing default value \"".concat(defValue, "\" instead"));
                value = defValue;
            }
            else {
                this.log('was not found in the environment, but is not required. returning undefined');
                // return undefined since variable is not required and
                // there's no default value provided
                return undefined;
            }
        }
        if (this.isRequired) {
            this.log('verifying variable value is not an empty string');
            // Need to verify that required variables aren't just whitespace
            if (value.trim().length === 0) {
                throw this.createError('is a required variable, but its value was empty');
            }
        }
        if (this.isBase64) {
            this.log('verifying variable is a valid base64 string');
            if (!value.match(base64Regex)) {
                throw this.createError('should be a valid base64 string if using convertFromBase64');
            }
            this.log('converting from base64 to utf8 string');
            value = Buffer.from(value, 'base64').toString();
        }
        try {
            return parser(value);
        }
        catch (e) {
            throw this.createError(e.message);
        }
    };
    Variable.prototype.usingAccessor = function (ext, args) {
        var _this = this;
        var value = this.asString();
        if (typeof value === 'string') {
            return ext(value, function (s) { return _this.createError(s); }, args);
        }
        else {
            return value;
        }
    };
    Variable.prototype.required = function (required) {
        if (required || required === undefined) {
            this.isRequired = true;
            return this;
        }
        else {
            this.isRequired = false;
            return this;
        }
    };
    Variable.prototype.asString = function () {
        return this.getValue(function (s) { return s; });
    };
    Variable.prototype.asArray = function (delimiter) {
        if (delimiter === void 0) { delimiter = ','; }
        return this.getValue(function (s) { return accessors_1.accessors.asArray(s, delimiter); });
    };
    Variable.prototype.asBoolStrict = function () {
        return this.getValue(accessors_1.accessors.asBoolStrict);
    };
    Variable.prototype.asBool = function () {
        return this.getValue(accessors_1.accessors.asBool);
    };
    // TODO: fix to a constrained return type based on allowed values
    Variable.prototype.asEnum = function (validValues) {
        return this.getValue(function (s) { return accessors_1.accessors.asEnum(s, validValues); });
    };
    Variable.prototype.asFloat = function () {
        return this.getValue(function (s) { return accessors_1.accessors.asFloat(s); });
    };
    Variable.prototype.asFloatNegative = function () {
        return this.getValue(accessors_1.accessors.asFloatNegative);
    };
    Variable.prototype.asFloatPositive = function () {
        return this.getValue(accessors_1.accessors.asFloatPositive);
    };
    Variable.prototype.asInt = function () {
        return this.getValue(accessors_1.accessors.asInt);
    };
    Variable.prototype.asIntNegative = function () {
        return this.getValue(accessors_1.accessors.asIntNegative);
    };
    Variable.prototype.asIntPositive = function () {
        return this.getValue(accessors_1.accessors.asIntPositive);
    };
    Variable.prototype.asJson = function () {
        return this.getValue(function (s) { return accessors_1.accessors.asJson(s); });
    };
    Variable.prototype.asJsonArray = function () {
        return this.getValue(accessors_1.accessors.asJsonArray);
    };
    Variable.prototype.asJsonObject = function () {
        return this.getValue(accessors_1.accessors.asJsonObject);
    };
    Variable.prototype.asPortNumber = function () {
        return this.getValue(accessors_1.accessors.asPortNumber);
    };
    Variable.prototype.asRegExp = function (flags) {
        return this.getValue(function (s) { return accessors_1.accessors.asRegExp(s, flags); });
    };
    Variable.prototype.asUrlObject = function () {
        return this.getValue(accessors_1.accessors.asUrlObject);
    };
    Variable.prototype.asUrlString = function () {
        return this.getValue(accessors_1.accessors.asUrlString);
    };
    return Variable;
}());
exports.Variable = Variable;
